LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;

entity FFTController is

	port (
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		nReset : in std_logic;
		
		ECG_Data : in std_logic_vector(7 downto 0);
		ECG_Clock : in std_logic;
		DataValid : in std_logic;
		
	
		
		RealOutData : out std_logic_vector (7 downto 0);
		RealExp : out std_logic_vector (5 downto 0);
		
		ImagOutData : out std_logic_vector (7 downto 0);
		sink_eopx : out std_logic;
		
		DataOutGood : out std_logic

	);

end FFTController;


architecture RTL of FFTController is

signal		clk	: STD_LOGIC;
signal		reset_n	:  STD_LOGIC;
signal		inverse	:  STD_LOGIC;
signal		sink_valid	:  STD_LOGIC;
signal		sink_sop	: STD_LOGIC;
signal		sink_eop	: STD_LOGIC;
signal		sink_real	: STD_LOGIC_VECTOR (7 DOWNTO 0);
signal		sink_imag	: STD_LOGIC_VECTOR (7 DOWNTO 0);
signal		sink_error	: STD_LOGIC_VECTOR (1 DOWNTO 0);
signal		source_ready	: STD_LOGIC;
signal		sink_ready	:  STD_LOGIC;
signal		source_error	: STD_LOGIC_VECTOR (1 DOWNTO 0);
signal		source_sop	: STD_LOGIC;
signal		source_eop	: STD_LOGIC;
signal		source_valid	: STD_LOGIC;
signal		source_exp	:  STD_LOGIC_VECTOR (5 DOWNTO 0);
signal		source_real	:  STD_LOGIC_VECTOR (7 DOWNTO 0);
signal		source_imag	: STD_LOGIC_VECTOR (7 DOWNTO 0);	
signal 		SourceContReady :  std_logic;
--signal 		DataOutGood : std_logic;
	
signal FFT_Clock : std_logic; -- derived from the ECG_clock, 

--Partition the control of the Avalon interface to seperate sink and source interfaces

component FFTSinkController is
port
(
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		nReset : in std_logic;
		
		ECG_Clock : in std_logic;  --sample clock
		DataValid : in std_logic; -- system says its ready
		sink_ready : in std_logic;  --FFT says its ready
		
		sink_valid : out std_logic;  --is input to FFT from system
		sink_sop : out std_logic;  -- inform FFT is start of packet
		sink_eop : out std_logic; -- inform FFT is end of packet
		FFT_Clock : out std_logic; -- derived from the ECG_clock, 
		
		source_error	: in STD_LOGIC_VECTOR (1 DOWNTO 0);
		sink_error	: out STD_LOGIC_VECTOR (1 DOWNTO 0)
		
			
);
end component;

component FFTSourceController is
port
(
			Clock50 : in std_logic;  -- might get a faster ext clock later!
			nReset : in std_logic;
			
			ECG_Clock : in std_logic;
			DataValid : in std_logic;
			source_valid : in std_logic;
			source_sop	: in STD_LOGIC;
			source_eop	: in STD_LOGIC;
		
			SourceContReady : out std_logic;
			DataOutGood : out std_logic
);
end component;


component FFTBlock is

PORT (
		clk	: IN STD_LOGIC;
		reset_n	: IN STD_LOGIC;
		inverse	: IN STD_LOGIC;
		sink_valid	: IN STD_LOGIC;
		sink_sop	: IN STD_LOGIC;
		sink_eop	: IN STD_LOGIC;
		sink_real	: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		sink_imag	: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		sink_error	: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		source_ready	: IN STD_LOGIC;
		sink_ready	: OUT STD_LOGIC;
		source_error	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		source_sop	: OUT STD_LOGIC;
		source_eop	: OUT STD_LOGIC;
		source_valid	: OUT STD_LOGIC;
		source_exp	: OUT STD_LOGIC_VECTOR (5 DOWNTO 0);
		source_real	: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		source_imag	: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);

end component;


	begin
	inverse <= '0';
	sink_imag <= "00000000";
	sink_eopx<=sink_eop;
	
	AvalonSink: FFTSinkController
	port map 
	(

		Clock50	=> Clock50,
	
		nReset => nReset,
		
		ECG_Clock  => ECG_Clock,  --sample clock
		
		DataValid  =>DataValid  , -- system says its ready
		sink_ready  => sink_ready,  --FFT says its ready
		
		sink_valid  => sink_valid,  --is input to FFT from system
		sink_sop  =>sink_sop ,  -- inform FFT is start of packet
		sink_eop  =>sink_eop , -- inform FFT is end of packet
		FFT_Clock  =>FFT_Clock,  -- derived from the ECG_clock, 
		source_error => source_error,
		sink_error=> sink_error
	
	);
----------------------------------------
-----------------------------------------

	AvalonSource : FFTSourceController
	port map
	(
		Clock50 => Clock50,  -- might get a faster ext clock later!
		nReset => nReset,
			
		ECG_Clock => ECG_Clock,
		DataValid =>DataValid ,
		source_valid => source_valid,
		source_sop => source_sop,
		source_eop => source_eop,
		SourceContReady => SourceContReady,	
		DataOutGood => DataOutGood
	);
	
	---
	ThisFFT : FFTBlock 
	port map
	(
		clk	=> FFT_Clock,
		reset_n	=> nReset,
		inverse	=> inverse,
		sink_valid	=>sink_valid ,
		sink_sop	=> sink_sop,
		sink_eop	=> sink_eop,
		sink_real	=> ECG_Data,
		sink_imag	=> sink_imag,
		sink_error	=> sink_error,
		source_ready	=> SourceContReady,
		sink_ready	=> sink_ready,
		source_error	=> source_error ,
		source_sop	=> source_sop,
		source_eop	=> source_eop,
		source_valid	=> source_valid,
		source_exp	=> RealExp,
		source_real	=> RealOutData,
		source_imag	=> ImagOutData
	
	);
	
	

end RTL;	