LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;

entity FFTSinkController is
	port (
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		nReset : in std_logic;	
		ECG_Clock : in std_logic;  --sample clock	
		DataValid : in std_logic; -- system says its ready
		sink_ready : in std_logic;  --FFT says its ready	
		sink_valid : out std_logic;  --is input to FFT from system
		sink_sop : out std_logic;  -- inform FFT is start of packet
		sink_eop : out std_logic; -- inform FFT is end of packet
		FFT_Clock : out std_logic; -- derived from the ECG_clock, 
		source_error	: in STD_LOGIC_VECTOR (1 DOWNTO 0);
		sink_error	: out STD_LOGIC_VECTOR (1 DOWNTO 0)
	);
end FFTSinkController;

architecture RTL of FFTSinkController is

type FFT_SinkState is (InReset,NotReady,Ready,PrepareFirstPacket,SetSop,FirstByte,NextByteLow,
								NextByteHi,WaitEop,SetEop,LastByte);
								
signal CurrentState : FFT_SinkState;
signal NextState : FFT_SinkState;
signal PacketCount : std_logic_vector (6 downto 0);
signal PacketCountFlag : std_logic;


	begin
	
	FFT_Clock<=ECG_Clock;
	sink_error <= "00";
--------------------------------------------------
--------------------------------------------------
	setSinkValid : process(nReset,DataValid,ECG_Clock)
	
		begin 
			if (nReset = '0') then
				sink_valid <='0';
			elsif (rising_edge (ECG_Clock)) then
				sink_valid <= DataValid;
			end if;
		end process;
		
	------------------------------------------
	-------------------------------------------
	
--------------------------------------------------
--------------------------------------------------
	Do_sink_sop : process(nReset,DataValid,PacketCount,ECG_Clock)
	
		begin 
			if (nReset = '0') then
				sink_sop <='0';
			elsif (rising_edge (ECG_Clock)) then
				if ((PacketCount= "0000000") and (DataValid = '1') and (sink_ready = '1')) then
					sink_sop <= '1';
				else 
					sink_sop <= '0';
				end if;
			end if;
		end process;	
------------------------------------------
-------------------------------------------
	Do_sink_eop : process(nReset,ECG_Clock,PacketCount)
	
		begin 
			if (nReset = '0') then
				sink_eop <='0';
			elsif (rising_edge (ECG_Clock)) then
				if ((PacketCount= "0111111") and (DataValid = '1') and (sink_ready = '1'))   then
					sink_eop <= '1';
				else 
					sink_eop <= '0';
				end if;
			end if;
		end process;		
	
--------------------------------------------------
--------------------------------------------------
	Do_DataCount : process(nReset,ECG_Clock)
	
		begin 
			if (nReset = '0') then
				PacketCount <="0000000";
			elsif (  (rising_edge (ECG_Clock)) and (DataValid = '1') and (sink_ready = '1'))  then
				PacketCount <= PacketCount + '1';
				if (PacketCount= "0111111") then
					PacketCount <="0000000";
				end if;
			end if;
		end process;	
------------------------------------------
-------------------------------------------	    
		
	----------------------


end RTL;	