LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 


entity Samos is 

port (
				Clock50 : in std_logic;  -- might get a faster ext clock later!
				nReset : in std_logic;
				ECG_Clock : in std_logic;
				
				DataGood : in std_logic;
				RealIn : in std_logic_vector (7 downto 0);
				ImagIn : in std_logic_vector (7 downto 0);
				
				ExpIn : in std_logic_vector (5 downto 0);
				
				ResultReady : out std_logic;
				ResultOut : out std_logic_vector (8 downto 0);
				ExpOut : out std_logic_vector (5 downto 0)
				
			);
			
	end Samos;
	
	
	architecture RTL of Samos is
	
	
	-- signals
	
	signal SquareThis : integer range 0 to 255; -- input to multiplier
	signal ThisSquared : integer range 0 to 65535;
	
	signal RealSquared : integer range 0 to 65535;
	signal ImagSquared : integer range 0 to 65535;
	
	signal ResultSquared : integer range 0 to 131071; -- this is radicand as integer
	
	signal ResultSquaredVector : std_logic_vector (16 downto 0);  -- this is radicand as vector ready to find root
	
	signal ResultVector : std_logic_vector (8 downto 0);  -- 
	
	signal RealReg : std_logic_vector (7 downto 0);
	signal ImagReg : std_logic_vector (7 downto 0);
	
	
	
	type PythagorasState is (InReset,WaitData,Debug0,SquareReal,Debug1,SquareImag,Debug2,AddThem,Debug3,GetRoot,Debug4,WaitResult,Debug5,PresentResult,Debug6);
	
	signal CurrentState : PythagorasState;
	signal NextState : PythagorasState;
	
	
	-- signals to interface to square root
	
	signal			Square :  std_logic_vector (16 downto 0);  --sample clock	
	signal			SquareStrobeIn :  std_logic; 			
	signal			Root	:  STD_LOGIC_VECTOR (8 DOWNTO 0); 
	signal			RootReady :  std_logic;
	
	-- component-
	component SquareRoot is
	port (
				Clock50 : in std_logic;  -- might get a faster ext clock later!
				nReset : in std_logic;	
				Square : in std_logic_vector (16 downto 0);  --sample clock	
				SquareStrobeIn : in std_logic; 		
				Root	: out STD_LOGIC_VECTOR (8 DOWNTO 0); 
				RootReady : out std_logic
			);
		
end Component;

	begin
	ExpOut <= ExpIn;
	-------------------------------------
	-- clock data in  on rising_edge of ECG_Clock
	-------------------------------------
	CaptureData : process (nReset,ECG_Clock)
	begin
			if (nReset ='0') then
				RealReg <= X"00";
				ImagReg <= X"00";
				
			elsif (rising_edge (ECG_Clock)) then
				RealReg <= RealIn;
				ImagReg <= ImagIn;
				
			end if;
	end process;
	-----------------------------------------
	-----------------------------------------
	--Clock data out on rising edge of main clock
	-- there is something else for it to validate against
	---------------------------------------
	ClockDataOut:process (nReset,Clock50)
	begin
		if ((nReset ='0') or (DataGood  ='0'))then
			ResultOut <="000000000";
		elsif (rising_edge (Clock50)) then
			ResultOut <= Root;
		end if;
	end process;
	----------------------------------------------
	-------------------
	-- local multiplier used to square the 'sides'
	------------------
	Multiplier : process (SquareThis)
	begin
		ThisSquared <= SquareThis*SquareThis;
	end process;
	------------------------------------------
	-- and thats it!!
	-------------------------------------------
	
	-----------------------------------------------
-----------------------------------------------
	StateReset:process (nReset,Clock50)
	begin
		if (nReset ='0') then 
			CurrentState <= InReset;
	   elsif (rising_edge (Clock50)) then
			CurrentState <= NextState;
		end if;
	end process;
------------------------------------------------
------------------------------------------------

	

process (CurrentState,DataGood,RootReady,ECG_Clock)

begin
--(InReset,WaitData,Debug0,SquareReal,Debug1,SquareImag,Debug2,AddThem,Debug3,GetRoot,Debug4,WaitResult,Debug5,PresentResult,Debug6);
	
	
	case CurrentState is
	------------------------
		when InReset => 
----------------------
				NextState <= WaitData;
	 ------------------------
		when WaitData => 
				if ((DataGood = '1') and (ECG_Clock = '1')) then
					NextState <= Debug0;
				else 
					NextState <= WaitData;
				end if;
	 ------------------------
		when Debug0 => 
				NextState <= SquareReal;
	 ------------------------
		when SquareReal => 
				NextState <= Debug1;
	 ------------------------
		when Debug1 =>
			NextState <= SquareImag;	
    ------------------------
	 	when SquareImag =>
			NextState <= Debug2;	
    ------------------------
	 	when Debug2 =>
			NextState <= AddThem;	
    ------------------------
	 	when AddThem =>
			NextState <= Debug3;	
    ------------------------
	  ------------------------
		when Debug3 => 
				NextState <= GetRoot;
	 ------------------------
		when GetRoot =>
			NextState <= Debug4;	
    ------------------------
	 	when Debug4 =>
			NextState <= WaitResult;	
    ------------------------
	 	when WaitResult =>
			if (RootReady = '1') then
				NextState <= Debug5;
			else 
				NextState <= WaitResult;
			end if;
		------------------------
	 	when Debug5 =>
			NextState <= PresentResult;	
    ------------------------	
    ------------------------
	 	when PresentResult =>
			NextState <= Debug6;	
	------------------------
	 	when Debug6 =>
			if (RootReady = '0') then
				NextState <= WaitData;	
			else
				NextState <= Debug6;
			end if;
	------------------------
		end case;
	end process;
-----------------------------------
-----------------------------------
process (CurrentState,Clock50)

begin

	if (rising_edge (Clock50)) then
	case CurrentState is
		------------------------
		when InReset => 
----------------------
			ResultReady<= '0';
	 ------------------------
		when WaitData => 
				
	 ------------------------
		when Debug0 => 
			SquareThis <= to_integer(unsigned(RealReg));
	 ------------------------
		when SquareReal => 
			 RealSquared <= ThisSquared;
	 ------------------------
		when Debug1 =>
			SquareThis <= to_integer(unsigned(ImagReg));
    ------------------------
	 	when SquareImag =>
			ImagSquared <= ThisSquared;
    ------------------------
	 	when Debug2 =>
			
    ------------------------
	 	when AddThem =>
			ResultSquared <= ImagSquared + RealSquared;
    ------------------------
	  ------------------------
		when Debug3 => 
				
			Square <= std_logic_vector(to_unsigned(ResultSquared,17));
	 ------------------------
		when GetRoot =>
			SquareStrobeIn <='1';
			
    ------------------------
	 	when Debug4 =>
			SquareStrobeIn <='0';
		 ------------------------
	 	when WaitResult =>	
    ------------------------
	 	when Debug5 =>		
    ------------------------
	 	when PresentResult =>
			--ResultOut <= Root;
	------------------------
	 	when Debug6 =>
			ResultReady<= '1';  --
	------------------------
		
		end case;
	end if;
	end process;

SqrtInst : SquareRoot

port map

(
		Clock50 => Clock50,   -- might get a faster ext clock later!
		nReset => nReset,	
		Square => Square,  --sample clock	
		SquareStrobeIn => SquareStrobeIn,			
		Root	=> Root,
		RootReady => RootReady
);
	
	
	
	end RTL;
	
				