LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 


entity ScopeDisplayBox is 

port (

	Clock50	: in std_logic;
		nReset : in std_logic;

	-- the input data is addressed like this,
	-- for easy 'face to math block
	-- concated to plain  address later
		ScopeAmplitude : in std_logic;
			ScopeStrobe : in std_logic;  -- strobe the value in. might not be totally necessary
			ScopeSweep : in std_logic;  -- probably only as an overide, should be automatic?
			ScopeReset : in std_logic; -- as opposed to global, might use for pause/restart
		
	-----------------------------------------------
	
	-- interface to VGA - takes all the 
	-- Hcount and Vcount that is thrown 
	-- at it, but only returns data for relevant
	-- area so is quite custom
		Horizontal : in integer range 0 to 1023;  -- the same as the Hcount in block above
		Vertical : in integer range 0 to 1023;   -- the same as Vcount in block above
	-- dont forget, horizontal is actually counting the columns (timeslots)
	-- and vertical the rows 
		ScopeRedOut : out std_logic_vector (3 downto 0);
		ScopeGreenOut : out std_logic_vector (3 downto 0);
		ScopeBlueOut : out std_logic_vector (3 downto 0);
		ScopeColourValid : out std_logic
	
	);
	
	end ScopeDisplayBox;
	
	
	
	architecture RTL of ScopeDisplayBox is
	
	
	
	signal Awraddress : std_logic_vector (14 downto 0);
	signal Ardaddress : std_logic_vector (14 downto 0);
	
	signal ScopeDispValid : std_logic;
	

	
	--debug only--debug only--debug only
	signal wrcount : std_logic_vector (15 downto 0);  --debug only
	
	
	signal DisplayInit: std_logic;

	
	
	signal ScopeDisplayValid : std_logic;
	
	signal ScopeByte : std_logic_vector (0 downto 0);
	signal ScopeVal : std_logic_vector (0 downto 0);
	signal Scoperdaddress : std_logic_vector(14 downto 0);	
	signal Scopewraddress : std_logic_vector(14 downto 0);	
	signal ScopeWriteToRam : std_logic;
	
	
	signal ScopeHorizontalInterim : integer range 0 to 1023; -- used to derive ram address from vga counts
	signal ScopeVerticalInterim : integer range 0 to 1023;   -- used to derive ram address from vga counts
	signal ScopeHorizontalVector : std_logic_vector (10 downto 0);--debug only
	signal ScopeVerticalVector : std_logic_vector (10 downto 0);--debug only
	signal ScopeScrollOffset : std_logic_vector(8 downto 0);
	signal ScopeRamLowOrderAddress : std_logic_vector (8 downto 0);
	
	signal ScopeRamHighOrderAddress : std_logic_vector (5 downto 0);
	
signal ScopeRamLowOrderAddressWr : std_logic_vector (8 downto 0);
	
	signal ScopeRamHighOrderAddressWr : std_logic_vector (5 downto 0);
	
	signal ScopeScrollEnable : std_logic;
		
		component ScopeRam is 
	
		PORT
		(
			clock		: IN STD_LOGIC  := '1';
			data		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			rdaddress		: IN STD_LOGIC_VECTOR (14 DOWNTO 0);
			wraddress		: IN STD_LOGIC_VECTOR (14 DOWNTO 0);
			wren		: IN STD_LOGIC  := '0';
			q		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
		);
		END component;
		
		
begin
-- lash up test

process (nReset,Clock50,DisplayInit)

begin
	if (nReset ='0') then 	
		wrcount <= X"0000";	
	elsif ((rising_edge (Clock50)) and (DisplayInit = '0')) then
		wrcount <= wrcount+'1';	
	end if;
end process;

----------------------------------------------------


process (nReset,wrcount)
	begin
		if (nReset = '0') then
			DisplayInit <= '0';
		--elsif (wrcount = X"0010") then 
		--	DisplayInit <= '1';
			else DisplayInit <= '1';
		end if;
end process;


----------------------------------------------------
process (nReset,ScopeStrobe,ScopeSweep) --point to the 'Y' co ordinat
begin
	if ((nReset='0') or (ScopeSweep= '1')) then
		ScopeRamHighOrderAddressWr <= "111111";  
	elsif ((rising_edge(ScopeStrobe) and (DisplayInit='1') ))then
		ScopeRamHighOrderAddressWr <= ScopeRamHighOrderAddressWr -'1';
	end if;
end process;	
-------------------------------------------


process (nReset,ScopeSweep)
begin
	if (nReset='0')then
		ScopeScrollOffset <= "000000000";
		ScopeRamLowOrderAddressWr <= "111111111";
	elsif (rising_edge (ScopeSweep)) then
		if (ScopeScrollEnable = '1') then
			ScopeScrollOffset <= ScopeScrollOffset +'1';
				if ScopeScrollOffset = "111111110" then
				 ScopeScrollOffset <="000000000";
				end if;
		end if;
		ScopeRamLowOrderAddressWr <=ScopeRamLowOrderAddressWr + '1';
	end if;
end process;

----------------------------------------------------------
process (nReset,Awraddress)
begin
	if (nReset='0')then
		ScopeScrollEnable<='0';
	elsif (ScopeRamLowOrderAddressWr = "000000000") then
		ScopeScrollEnable<='1';
		
	end if;
end process;


	
Awraddress (14 downto 9) <= ScopeRamHighOrderAddressWr;
Awraddress (8 downto 0)	<= ScopeRamLowOrderAddressWr;

process (nReset,wrcount,DisplayInit,Clock50)

begin
		if 
			(DisplayInit ='0') then
			Scopewraddress <= wrcount(14 downto 0);
			ScopeWriteToRam <= (not Clock50);
			ScopeVal(0)<= '0';		
		else 
			Scopewraddress <= Awraddress;
			ScopeWriteToRam <= ScopeStrobe;
			ScopeVal(0) <= ScopeAmplitude;  
		end if;
end process;


process (nReset,Horizontal,Vertical)

begin

	if (nReset = '0' ) then
		ScopeDisplayValid <= '0';
	 elsif (  ( (Horizontal > 64)and(Horizontal < 576 )) and ((Vertical >320) and (Vertical < 384)) ) then
		ScopeDisplayValid <= '1';
	 else 
		ScopeDisplayValid <= '0';
	end if;
end process;


-- somehow get the data out

process (nReset,ScopeDisplayValid)

begin

	if (nReset ='0') then

		ScopeRedOut <= "0000";
		ScopeGreenOut <= "0000";
		ScopeBlueOut <= "0000";
		
	else
	
		if ( (ScopeDisplayValid = '1') and (DisplayInit ='1')) then
	
			ScopeRedOut <= "0000";
			ScopeGreenOut <= "0000";
			ScopeBlueOut(3) <= ScopeByte(0);	
			ScopeBlueOut(2) <= ScopeByte(0);	
			ScopeBlueOut(1) <= ScopeByte(0);	
			ScopeBlueOut(0) <= ScopeByte(0);	 -- bit lame, find out how to use 'other'
		else
			ScopeRedOut <= "0000";
			ScopeGreenOut <= "0000";
			ScopeBlueOut <= "0000";
		
	end if;
end if;
	
end process;
	

-- Scope only
ScopeHorizontalInterim <= Horizontal-64;--debug only
ScopeVerticalInterim <= Vertical-320;--debug only
ScopeHorizontalVector <=  std_logic_vector(to_unsigned(ScopeHorizontalInterim,11));--debug only
ScopeVerticalVector <=  std_logic_vector(to_unsigned(ScopeVerticalInterim,11));--debug only
Scoperdaddress(14 downto 9)<= ScopeVerticalVector(5 downto 0);
ScopeRamLowOrderAddress <= ScopeHorizontalVector(8 downto 0)+ ScopeScrollOffset;
Scoperdaddress (8 downto 0)<= ScopeRamLowOrderAddress ;


-- common interface 

--DisplayReady <= DisplayInit;



		
		SourceMon : ScopeRam
		
		PORT MAP 
		(
			clock	 => Clock50,
			data	 => ScopeVal,  -- put this back when finished
			rdaddress=>Scoperdaddress,	
			wraddress=>Scopewraddress,	
			wren	 => ScopeWriteToRam,
			q	 => ScopeByte
		);
		
		
ScopeColourValid <=  ScopeDisplayValid;
		
		
end RTL;

	
