LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 

-- system emulator, 

entity ScopeSource is

	port
	(
		Clock50	: in std_logic;
		nReset : in std_logic;
		Clock1K : in std_logic;
		Hold : in std_logic;
		ScopeHold : in std_logic;  -- I know!! a hold was already provisioned. Trust me!
		--NullData : in std_logic;
		-- currently just outputs stuff
		ScopeAmplitude : out std_logic;
		ScopeStrobe : out std_logic;  -- strobe the value in. might not be totally necessary
		ScopeSweep : out std_logic;  -- probably only as an overide, should be automatic?
		ScopeReset : out std_logic; -- as opposed to global, might use for pause/restart
		RawData : out std_logic_vector (7 downto 0)
		
	);
end ScopeSource;

architecture RTL of ScopeSource is


type Word5Array is array (0 to 255) of integer ;
signal ScopeLut : Word5Array;
signal index : integer range 0 to 255;
signal SeekAmplitude: integer;

begin





	
process (nReset,Clock1K)
	begin
		if (nReset ='0') then 
			RawData <= "00000000";
	   elsif (rising_edge (Clock1K)) then
			RawData <= std_logic_vector(to_unsigned(SeekAmplitude,8));	
		end if;
	end process;
	
			
	
	process (nReset,Clock1K)
	begin
		if (nReset ='0') then 
			index <= 0;
	   elsif (falling_edge (Clock1K)) then
			index <=index+1;
			if (index = 254) then
				index <= 0;
			end if;
		end if;
	end process;
	
SeekAmplitude<=ScopeLut(index);
	
	

ScopeLut(	0	)<=	0	;
ScopeLut(	1	)<=	3	;
ScopeLut(	2	)<=	6	;
ScopeLut(	3	)<=	9	;
ScopeLut(	4	)<=	12	;
ScopeLut(	5	)<=	15	;
ScopeLut(	6	)<=	18	;
ScopeLut(	7	)<=	20	;
ScopeLut(	8	)<=	22	;
ScopeLut(	9	)<=	25	;
ScopeLut(	10	)<=	26	;
ScopeLut(	11	)<=	28	;
ScopeLut(	12	)<=	29	;
ScopeLut(	13	)<=	30	;
ScopeLut(	14	)<=	31	;
ScopeLut(	15	)<=	31	;
ScopeLut(	16	)<=	31	;
ScopeLut(	17	)<=	31	;
ScopeLut(	18	)<=	31	;
ScopeLut(	19	)<=	30	;
ScopeLut(	20	)<=	29	;
ScopeLut(	21	)<=	27	;
ScopeLut(	22	)<=	25	;
ScopeLut(	23	)<=	23	;
ScopeLut(	24	)<=	21	;
ScopeLut(	25	)<=	19	;
ScopeLut(	26	)<=	16	;
ScopeLut(	27	)<=	13	;
ScopeLut(	28	)<=	10	;
ScopeLut(	29	)<=	7	;
ScopeLut(	30	)<=	4	;
ScopeLut(	31	)<=	1	;
ScopeLut(	32	)<=	-2	;
ScopeLut(	33	)<=	-6	;
ScopeLut(	34	)<=	-9	;
ScopeLut(	35	)<=	-12	;
ScopeLut(	36	)<=	-15	;
ScopeLut(	37	)<=	-17	;
ScopeLut(	38	)<=	-20	;
ScopeLut(	39	)<=	-23	;
ScopeLut(	40	)<=	-25	;
ScopeLut(	41	)<=	-27	;
ScopeLut(	42	)<=	-28	;
ScopeLut(	43	)<=	-30	;
ScopeLut(	44	)<=	-31	;
ScopeLut(	45	)<=	-32	;
ScopeLut(	46	)<=	-32	;
ScopeLut(	47	)<=	-32	;
ScopeLut(	48	)<=	-32	;
ScopeLut(	49	)<=	-32	;
ScopeLut(	50	)<=	-31	;
ScopeLut(	51	)<=	-30	;
ScopeLut(	52	)<=	-29	;
ScopeLut(	53	)<=	-27	;
ScopeLut(	54	)<=	-25	;
ScopeLut(	55	)<=	-23	;
ScopeLut(	56	)<=	-21	;
ScopeLut(	57	)<=	-18	;
ScopeLut(	58	)<=	-15	;
ScopeLut(	59	)<=	-12	;
ScopeLut(	60	)<=	-9	;
ScopeLut(	61	)<=	-6	;
ScopeLut(	62	)<=	-3	;
ScopeLut(	63	)<=	0	;
ScopeLut(	64	)<=	3	;
ScopeLut(	65	)<=	6	;
ScopeLut(	66	)<=	9	;
ScopeLut(	67	)<=	12	;
ScopeLut(	68	)<=	15	;
ScopeLut(	69	)<=	18	;
ScopeLut(	70	)<=	21	;
ScopeLut(	71	)<=	23	;
ScopeLut(	72	)<=	25	;
ScopeLut(	73	)<=	27	;
ScopeLut(	74	)<=	28	;
ScopeLut(	75	)<=	30	;
ScopeLut(	76	)<=	30	;
ScopeLut(	77	)<=	31	;
ScopeLut(	78	)<=	31	;
ScopeLut(	79	)<=	31	;
ScopeLut(	80	)<=	31	;
ScopeLut(	81	)<=	31	;
ScopeLut(	82	)<=	30	;
ScopeLut(	83	)<=	28	;
ScopeLut(	84	)<=	27	;
ScopeLut(	85	)<=	25	;
ScopeLut(	86	)<=	23	;
ScopeLut(	87	)<=	21	;
ScopeLut(	88	)<=	18	;
ScopeLut(	89	)<=	16	;
ScopeLut(	90	)<=	13	;
ScopeLut(	91	)<=	10	;
ScopeLut(	92	)<=	7	;
ScopeLut(	93	)<=	3	;
ScopeLut(	94	)<=	0	;
ScopeLut(	95	)<=	-3	;
ScopeLut(	96	)<=	-6	;
ScopeLut(	97	)<=	-9	;
ScopeLut(	98	)<=	-12	;
ScopeLut(	99	)<=	-15	;
ScopeLut(	100	)<=	-18	;
ScopeLut(	101	)<=	-21	;
ScopeLut(	102	)<=	-23	;
ScopeLut(	103	)<=	-25	;
ScopeLut(	104	)<=	-27	;
ScopeLut(	105	)<=	-29	;
ScopeLut(	106	)<=	-30	;
ScopeLut(	107	)<=	-31	;
ScopeLut(	108	)<=	-32	;
ScopeLut(	109	)<=	-32	;
ScopeLut(	110	)<=	-32	;
ScopeLut(	111	)<=	-32	;
ScopeLut(	112	)<=	-32	;
ScopeLut(	113	)<=	-31	;
ScopeLut(	114	)<=	-30	;
ScopeLut(	115	)<=	-29	;
ScopeLut(	116	)<=	-27	;
ScopeLut(	117	)<=	-25	;
ScopeLut(	118	)<=	-23	;
ScopeLut(	119	)<=	-20	;
ScopeLut(	120	)<=	-18	;
ScopeLut(	121	)<=	-15	;
ScopeLut(	122	)<=	-12	;
ScopeLut(	123	)<=	-9	;
ScopeLut(	124	)<=	-6	;
ScopeLut(	125	)<=	-3	;
ScopeLut(	126	)<=	1	;
ScopeLut(	127	)<=	4	;
ScopeLut(	128	)<=	7	;
ScopeLut(	129	)<=	10	;
ScopeLut(	130	)<=	13	;
ScopeLut(	131	)<=	16	;
ScopeLut(	132	)<=	18	;
ScopeLut(	133	)<=	21	;
ScopeLut(	134	)<=	23	;
ScopeLut(	135	)<=	25	;
ScopeLut(	136	)<=	27	;
ScopeLut(	137	)<=	28	;
ScopeLut(	138	)<=	30	;
ScopeLut(	139	)<=	31	;
ScopeLut(	140	)<=	31	;
ScopeLut(	141	)<=	31	;
ScopeLut(	142	)<=	31	;
ScopeLut(	143	)<=	31	;
ScopeLut(	144	)<=	30	;
ScopeLut(	145	)<=	29	;
ScopeLut(	146	)<=	28	;
ScopeLut(	147	)<=	27	;
ScopeLut(	148	)<=	25	;
ScopeLut(	149	)<=	23	;
ScopeLut(	150	)<=	20	;
ScopeLut(	151	)<=	18	;
ScopeLut(	152	)<=	15	;
ScopeLut(	153	)<=	12	;
ScopeLut(	154	)<=	9	;
ScopeLut(	155	)<=	6	;
ScopeLut(	156	)<=	3	;
ScopeLut(	157	)<=	0	;
ScopeLut(	158	)<=	-3	;
ScopeLut(	159	)<=	-7	;
ScopeLut(	160	)<=	-10	;
ScopeLut(	161	)<=	-13	;
ScopeLut(	162	)<=	-16	;
ScopeLut(	163	)<=	-18	;
ScopeLut(	164	)<=	-21	;
ScopeLut(	165	)<=	-23	;
ScopeLut(	166	)<=	-25	;
ScopeLut(	167	)<=	-27	;
ScopeLut(	168	)<=	-29	;
ScopeLut(	169	)<=	-30	;
ScopeLut(	170	)<=	-31	;
ScopeLut(	171	)<=	-32	;
ScopeLut(	172	)<=	-32	;
ScopeLut(	173	)<=	-32	;
ScopeLut(	174	)<=	-32	;
ScopeLut(	175	)<=	-32	;
ScopeLut(	176	)<=	-31	;
ScopeLut(	177	)<=	-30	;
ScopeLut(	178	)<=	-28	;
ScopeLut(	179	)<=	-27	;
ScopeLut(	180	)<=	-25	;
ScopeLut(	181	)<=	-22	;
ScopeLut(	182	)<=	-20	;
ScopeLut(	183	)<=	-17	;
ScopeLut(	184	)<=	-14	;
ScopeLut(	185	)<=	-11	;
ScopeLut(	186	)<=	-8	;
ScopeLut(	187	)<=	-5	;
ScopeLut(	188	)<=	-2	;
ScopeLut(	189	)<=	1	;
ScopeLut(	190	)<=	4	;
ScopeLut(	191	)<=	7	;
ScopeLut(	192	)<=	10	;
ScopeLut(	193	)<=	13	;
ScopeLut(	194	)<=	16	;
ScopeLut(	195	)<=	19	;
ScopeLut(	196	)<=	21	;
ScopeLut(	197	)<=	24	;
ScopeLut(	198	)<=	26	;
ScopeLut(	199	)<=	27	;
ScopeLut(	200	)<=	29	;
ScopeLut(	201	)<=	30	;
ScopeLut(	202	)<=	31	;
ScopeLut(	203	)<=	31	;
ScopeLut(	204	)<=	31	;
ScopeLut(	205	)<=	31	;
ScopeLut(	206	)<=	31	;
ScopeLut(	207	)<=	30	;
ScopeLut(	208	)<=	29	;
ScopeLut(	209	)<=	28	;
ScopeLut(	210	)<=	26	;
ScopeLut(	211	)<=	24	;
ScopeLut(	212	)<=	22	;
ScopeLut(	213	)<=	20	;
ScopeLut(	214	)<=	17	;
ScopeLut(	215	)<=	15	;
ScopeLut(	216	)<=	12	;
ScopeLut(	217	)<=	9	;
ScopeLut(	218	)<=	6	;
ScopeLut(	219	)<=	2	;
ScopeLut(	220	)<=	-1	;
ScopeLut(	221	)<=	-4	;
ScopeLut(	222	)<=	-7	;
ScopeLut(	223	)<=	-10	;
ScopeLut(	224	)<=	-13	;
ScopeLut(	225	)<=	-16	;
ScopeLut(	226	)<=	-19	;
ScopeLut(	227	)<=	-21	;
ScopeLut(	228	)<=	-24	;
ScopeLut(	229	)<=	-26	;
ScopeLut(	230	)<=	-28	;
ScopeLut(	231	)<=	-29	;
ScopeLut(	232	)<=	-30	;
ScopeLut(	233	)<=	-31	;
ScopeLut(	234	)<=	-32	;
ScopeLut(	235	)<=	-32	;
ScopeLut(	236	)<=	-32	;
ScopeLut(	237	)<=	-32	;
ScopeLut(	238	)<=	-32	;
ScopeLut(	239	)<=	-31	;
ScopeLut(	240	)<=	-29	;
ScopeLut(	241	)<=	-28	;
ScopeLut(	242	)<=	-26	;
ScopeLut(	243	)<=	-24	;
ScopeLut(	244	)<=	-22	;
ScopeLut(	245	)<=	-19	;
ScopeLut(	246	)<=	-17	;
ScopeLut(	247	)<=	-14	;
ScopeLut(	248	)<=	-11	;
ScopeLut(	249	)<=	-8	;
ScopeLut(	250	)<=	-5	;
ScopeLut(	251	)<=	-2	;
ScopeLut(	252	)<=	2	;
ScopeLut(	253	)<=	5	;
ScopeLut(	254	)<=	8	;
ScopeLut(	255	)<=	11	;







end RTL;
