LIBRARY ieee ;
USE ieee.std_logic_1164.all ;--
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 


entity SliceBuffer is 

port 
	(	
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		nReset : in std_logic;	
		Clock4K : in std_logic;
		Clock1K : in std_logic;  --sample clock
		RawData : in std_logic_vector (7 downto 0); -- 
		DataGood : in std_logic; -- inform this entity that data is valid
		sink_eop : in std_logic; -- informing FFT is end of packet, and here to reset window index
		Buffered_Data : out std_logic_vector (7 downto 0)  -- the data after window applied
			
	);
		
end SliceBuffer;

architecture RTL of SliceBuffer is

signal LocalReset : std_logic;
signal ReadAddress : std_logic_vector (6 downto 0);
signal data : std_logic_vector (7 downto 0);
signal q : std_logic_vector (7 downto 0);
signal WriteToRam : std_logic;
-- signals Ive added for this refactoring
signal NumDataBuffered : std_logic_vector (6 downto 0); -- just used in init
signal ReadyToRead : std_logic; -- flag that buffer has at least a packets worth
signal writerambuff : std_logic_vector (2 downto 0); 
signal ScopeReadAddress : std_logic_vector (6 downto 0); -- to implement later, the raw data 									
signal DataCountOut : std_logic_vector (6 downto 0); -- to track how much data sent out 																			
signal DataAbsReadAddr : std_logic_vector (6 downto 0); -- address used to fetch data from Ram
signal FFTinputStarted : std_logic; -- flag indicating that FFT is accepting data now
signal wraddress : std_logic_vector (6 downto 0);																		
-- signals pasted in from window
--type TableEntry is range 0 to 64;  --
type table64 is array (64 downto 0) of integer range 0 to 1024; 
signal HannTable : table64;
signal TableIndex : integer range 0 to 64; -- index is one more than it actually has to be 													  
signal result : integer;
signal interimresult : std_logic_vector (17 downto 0);
signal TheInput : integer;
signal Windowed_Data : std_logic_vector (7 downto 0);

-------------------------------
component SliceRam IS
	PORT
	(
		clock		: IN STD_LOGIC  := '1';
		data		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
		rdaddress		: IN STD_LOGIC_VECTOR (6 DOWNTO 0);
		wraddress		: IN STD_LOGIC_VECTOR (6 DOWNTO 0);
		wren		: IN STD_LOGIC  := '0';
		q		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
END component;

begin 

LocalReset <= nReset and DataGood;  --

----------------------------------------
----------------------------------------
--write to Ram here
----------------------------------------
----------------------------------------
-- Data clocked in on slow(er) Clock
DataIn : process (LocalReset,Clock1K)
	begin
		if (LocalReset ='0') then 
			data <= "00000000";
			wraddress <= "1111111";
			ReadyToRead <='0';
			NumDataBuffered <= "0000000";
	   elsif (rising_edge (Clock1K))  then
			data <= RawData;
			wraddress <= wraddress + '1';
			NumDataBuffered <= NumDataBuffered + '1';
			if (NumDataBuffered > "1000001") then
				ReadyToRead <= '1';
			end if;		
		end if;
	end process;
---------------------------------------
---------------------------------------
---------------------------------------
WriteRamClock : process (LocalReset,Clock1K,Clock50)
begin
	if (LocalReset = '0') then
		writerambuff <= "000";
	elsif (rising_edge (Clock50)) then 
			if ((Clock1K = '1') and (writerambuff < "100")) then
				writerambuff <= writerambuff + '1';
			else
				writerambuff <="000";
			end if;
	end if;
end process;
-----------------------------------------
WriteToRam <= writerambuff(1); -- clock it in with this
-----------------------------------------
-----------------------------------------
-- and thats that for writing
-------------
-- now take care of read address and hann table index
ReadBuffAddress : process (LocalReset,Clock4K,FFTinputStarted,ReadyToRead,DataAbsReadAddr,DataCountOut)
begin
	if (LocalReset = '0') then
		DataAbsReadAddr <= "0000000";
		DataCountOut <="0000000";
	elsif ((Rising_edge(Clock4K)) and (FFTinputStarted = '1') and (ReadyToRead = '1')) then	
		DataAbsReadAddr <= DataAbsReadAddr + '1';
		DataCountOut <= DataCountOut +'1';
		if (DataCountOut = "0111111") then
			DataAbsReadAddr <= DataAbsReadAddr - "0110000"; -- regress 48 points for 'overlap'
			DataCountOut <="0000000"; -- which can also be the index for the hann table!
		end if;
	end if;
end process;
----------------------------------
----------------------------------
 RamReadAddr : process (LocalReset,Clock4K,DataAbsReadAddr) 
 begin
	if (LocalReset = '0') then
		ReadAddress <= "0000000";
	elsif (falling_edge (Clock4K)) then
		ReadAddress <= DataAbsReadAddr;
	end if;
	
end process;
------------------------------------
-------------------------------------
--window/hann function stuff
-----------------------------------------
-- Data clocked in and out on rising edge of
-- Clock4K, OUTSDIDE of the state machine
-------------------------------------------
init : process (nReset,Clock4K)
	begin
		if (nReset ='0') then 
			Buffered_Data  <="00000000";
	   elsif (rising_edge (Clock4K))  then
			Buffered_Data  <=Windowed_Data;
			end if;
	end process;
------------------------------------------------------------------	
DoWindow:process (nReset,Clock50,TheInput,HannTable,TableIndex)
begin
	if (Rising_edge (Clock50)) then
		FFTinputStarted <='1';
		result <= TheInput*HannTable(TableIndex);	
	end if;	
end process;
----------------------------------------------------------------
TableIndex <= to_integer(unsigned(DataCountOut));
TheInput <= to_integer(signed(q));
interimresult <= std_logic_vector(to_signed(result,18));
Windowed_Data <= interimresult(17 downto 10);
------------------------------------------------------------------
SliceRamInst : SliceRam 
	PORT map
	(
		clock	=> Clock50,
		data	=>	data,
		rdaddress =>ReadAddress,
		wraddress =>wraddress,
		wren	=>	WriteToRam,
		q		=>q
	);
----------------------------------------
--- Look Up Table for Hann Function ----
------------------------------------------	
HannTable(	0	)<=	0	;
HannTable(	1	)<=	2	;
HannTable(	2	)<=	10	;
HannTable(	3	)<=	22	;
HannTable(	4	)<=	40	;
HannTable(	5	)<=	62	;
HannTable(	6	)<=	88	;
HannTable(	7	)<=	119	;
HannTable(	8	)<=	154	;
HannTable(	9	)<=	192	;
HannTable(	10	)<=	234	;
HannTable(	11	)<=	278	;
HannTable(	12	)<=	324	;
HannTable(	13	)<=	373	;
HannTable(	14	)<=	423	;
HannTable(	15	)<=	473	;
HannTable(	16	)<=	524	;
HannTable(	17	)<=	575	;
HannTable(	18	)<=	625	;
HannTable(	19	)<=	675	;
HannTable(	20	)<=	722	;
HannTable(	21	)<=	768	;
HannTable(	22	)<=	810	;
HannTable(	23	)<=	850	;
HannTable(	24	)<=	887	;
HannTable(	25	)<=	920	;
HannTable(	26	)<=	948	;
HannTable(	27	)<=	973	;
HannTable(	28	)<=	993	;
HannTable(	29	)<=	1008	;
HannTable(	30	)<=	1018	;
HannTable(	31	)<=	1023	;
HannTable(	32	)<=	1023	;
HannTable(	33	)<=	1018	;
HannTable(	34	)<=	1008	;
HannTable(	35	)<=	993	;
HannTable(	36	)<=	973	;
HannTable(	37	)<=	948	;
HannTable(	38	)<=	920	;
HannTable(	39	)<=	887	;
HannTable(	40	)<=	850	;
HannTable(	41	)<=	810	;
HannTable(	42	)<=	768	;
HannTable(	43	)<=	722	;
HannTable(	44	)<=	675	;
HannTable(	45	)<=	625	;
HannTable(	46	)<=	575	;
HannTable(	47	)<=	524	;
HannTable(	48	)<=	473	;
HannTable(	49	)<=	423	;
HannTable(	50	)<=	373	;
HannTable(	51	)<=	324	;
HannTable(	52	)<=	278	;
HannTable(	53	)<=	234	;
HannTable(	54	)<=	192	;
HannTable(	55	)<=	154	;
HannTable(	56	)<=	119	;
HannTable(	57	)<=	88	;
HannTable(	58	)<=	62	;
HannTable(	59	)<=	40	;
HannTable(	60	)<=	22	;
HannTable(	61	)<=	10	;
HannTable(	62	)<=	2	;
HannTable(	63	)<=	0	;
HannTable(	64	)<=	0	;
	
end RTL;