LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 


entity SpectrumDisplayBox is 

port (

		Clock50	: in std_logic;
		nReset : in std_logic;

	-- the input data is addressed like this,
	-- for easy 'face to math block
	-- concated to plain  address later
		-- system interface
		SpecAmplitude : in std_logic_vector (7 downto 0);
		SpecStrobe : in  std_logic;  -- strobe the value in. might not be totally necessary
		SpecSweep : in  std_logic;  -- probably only as an overide, should be automatic?
		SpecReset : in  std_logic; -- as opposed to global, might use for pause/restart
	
	-----------------------------------------------
	
	-- interface to VGA - takes all the 
	-- Hcount and Vcount that is thrown 
	-- at it, but only returns data for relevant
	-- area so is quite custom
		Horizontal : in integer range 0 to 1023;  -- the same as the Hcount in block above
		Vertical : in integer range 0 to 1023;   -- the same as Vcount in block above
	-- dont forget, horizontal is actually counting the columns (timeslots)
	-- and vertical the rows 
		SpecRedOut : out std_logic_vector (3 downto 0);
		SpecGreenOut : out std_logic_vector (3 downto 0);
		SpecBlueOut : out std_logic_vector (3 downto 0);
		SpecColourValid : out std_logic
	
	);
	
	end SpectrumDisplayBox;
	
	
	
	architecture RTL of SpectrumDisplayBox is
	
	
	signal Awraddress : std_logic_vector (12 downto 0);
	
	
	
	

	
	--debug only--debug only--debug only
	signal wrcount : std_logic_vector (15 downto 0);  --debug only
	
	
	signal DisplayInit: std_logic;

	
	
	signal SpecDisplayValid : std_logic;
	
	signal SpecByte : std_logic_vector (7 downto 0);
	signal SpecVal : std_logic_vector (7 downto 0);
	signal Specrdaddress : std_logic_vector(12 downto 0);	
	signal Specwraddress : std_logic_vector(12 downto 0);	
	signal SpecWriteToRam : std_logic;
	
	
	signal SpecHorizontalInterim : integer range 0 to 1023; -- used to derive ram address from vga counts
	signal SpecVerticalInterim : integer range 0 to 1023;   -- used to derive ram address from vga counts
	signal SpecHorizontalVector : std_logic_vector (10 downto 0);--debug only
	signal SpecVerticalVector : std_logic_vector (10 downto 0);--debug only
	signal SpecScrollOffset : std_logic_vector(8 downto 0);
	signal SpecRamLowOrderAddress : std_logic_vector (8 downto 0);
	
	signal SpecRamHighOrderAddress : std_logic_vector (4 downto 0);
	
signal SpecRamLowOrderAddressWr : std_logic_vector (7 downto 0);
	
	signal SpecRamHighOrderAddressWr : std_logic_vector (4 downto 0);
	
	signal SpecScrollEnable : std_logic;
	
	
	component DisplayRam is 
	
		PORT
		(
			clock		: IN STD_LOGIC  := '1';
			data		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
			rdaddress		: IN STD_LOGIC_VECTOR (12 DOWNTO 0);
			wraddress		: IN STD_LOGIC_VECTOR (12 DOWNTO 0);
			wren		: IN STD_LOGIC  := '0';
			q		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
		);
		END component;
		
		
		
begin
-- lash up test


process (nReset,Clock50,DisplayInit)

begin
	if (nReset ='0') then 	
		wrcount <= X"0000";	
	elsif ((rising_edge (Clock50)) and (DisplayInit = '0')) then
		wrcount <= wrcount+'1';	
	end if;
end process;

----------------------------------------------------


process (nReset,wrcount)
	begin
		if (nReset = '0') then
			DisplayInit <= '0';
			else DisplayInit <= '1';
		end if;
end process;
-----------------------------------------------

------------------------------------------------------
process (nReset,SpecStrobe,SpecSweep) --point to the 'Y' co ordinat
begin
	if ((nReset='0') or (SpecSweep= '1')) then
		SpecRamHighOrderAddressWr <= "11111";  
	elsif ((rising_edge(SpecStrobe) and (DisplayInit='1') ))then
		SpecRamHighOrderAddressWr <= SpecRamHighOrderAddressWr +'1';
	end if;
end process;	
---------------------------------------------	


process (nReset,SpecSweep)
begin
	if (nReset='0')then
	SpecScrollOffset <= "000000000";
		SpecRamLowOrderAddressWr <= X"FF";
	elsif (rising_edge (SpecSweep)) then
		if (SpecScrollEnable = '1') then
			SpecScrollOffset <= SpecScrollOffset +'1';
				if SpecScrollOffset = "111111111" then
					SpecScrollOffset <="000000000";
				end if;
		end if;
		SpecRamLowOrderAddressWr <=SpecRamLowOrderAddressWr + '1';
	end if;
end process;
------------------------------------
------------------------------------

process (nReset,SpecRamLowOrderAddressWr)
begin
	if (nReset='0')then
		SpecScrollEnable<='0';
	elsif (SpecRamLowOrderAddressWr = "00000000") then
		SpecScrollEnable<='1';
		
	end if;
end process;


	
Awraddress (12 downto 8) <= SpecRamHighOrderAddressWr(4 downto 0);
Awraddress (7 downto 0)	<= SpecRamLowOrderAddressWr(7 downto 0);

process (nReset,DisplayInit,Clock50,SpecStrobe)

begin
		if 
			(DisplayInit ='0') then
			Specwraddress <= wrcount(12 downto 0);
			SpecWriteToRam <= (not Clock50);
			SpecVal<= "00000000";		
		else 
			Specwraddress <= Awraddress;
			SpecWriteToRam <= SpecStrobe;
			SpecVal(7 downto 0) <= SpecAmplitude;  
		end if;
end process;


process (nReset,Horizontal,Vertical)

begin

	if (nReset = '0' ) then
		SpecDisplayValid <= '0';
	 elsif (  ( (Horizontal > 64)and(Horizontal < 576 )) and ((Vertical >20) and (Vertical < 276)) ) then
		SpecDisplayValid <= '1';
	 else 
		SpecDisplayValid <= '0';
	end if;
end process;


-- somehow get the data out

process (nReset,SpecDisplayValid,SpecByte,DisplayInit)

begin

	if (nReset ='0') then

		SpecRedOut <= "0000";
		SpecGreenOut <= "0000";
		SpecBlueOut <= "0000";
		
	else
	
		if ( (SpecDisplayValid = '1') and (DisplayInit ='1')) then
	
			SpecRedOut(3 downto 0) <= SpecByte (7 downto 4);
			SpecGreenOut(3 downto 0) <= SpecByte (7 downto 4);
			SpecBlueOut(3 downto 0)<= 	 SpecByte (7 downto 4);-- bit lame, find out how to use 'other'
		else
			SpecRedOut <= "0000";
			SpecGreenOut <= "0000";
			SpecBlueOut <= "0000";
		
	end if;
end if;
	
end process;
	


--my_slv <= std_logic_vector(to_unsigned(my_integer, my_slv'length)); -- if 



-- Spec only
SpecHorizontalInterim <= Horizontal-64;--debug only
SpecVerticalInterim <= Vertical-20;--debug only

SpecHorizontalVector <=  std_logic_vector(to_unsigned(SpecHorizontalInterim,11));--debug only
SpecVerticalVector <=  std_logic_vector(to_unsigned(SpecVerticalInterim,11));--debug only

Specrdaddress(12 downto 8)<= SpecVerticalVector(7 downto 3);
SpecRamLowOrderAddress <= SpecHorizontalVector(8 downto 0);--+ SpecScrollOffset ;
Specrdaddress (7 downto 0)<= SpecRamLowOrderAddress(8 downto 1)+ SpecScrollOffset (7 downto 0);

SpecColourValid <= SpecDisplayValid;	
	

-- instantiate spectrogram d
	SpectRam : DisplayRam

		PORT MAP 
		(
			clock	 => Clock50,
			data	 => SpecVal,  
			rdaddress=>Specrdaddress ,	
			wraddress=>Specwraddress,	
			wren	 => SpecWriteToRam,
			q	 => SpecByte
		);
		
		
		
		

end RTL;

	
