LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 

-- system emulator, 

entity SpectrumGenerator is

	port
	(
		Clock50	: in std_logic;
		nReset : in std_logic;
		Clock4K : in std_logic;
		Clock1K : in std_logic;
		Hold : in std_logic;
		RawData : in std_logic_vector(7 downto 0);
		--NullData : in std_logic;
		-- currently just outputs stuff
		SpecAmplitude : out std_logic_vector (7 downto 0);
		SpecStrobe : out std_logic;  -- strobe the value in. might not be totally necessary
		SpecSweep : out std_logic;  -- probably only as an overide, should be automatic?
		SpecReset : out std_logic; -- as opposed to global, might use for pause/restart
		ScopeHold : out std_logic; -- so the spectrum can hold the scope back until 
										  -- some spectral data is on its way
		
		Debug1 : out Std_logic									 
	);
end SpectrumGenerator;

architecture RTL of SpectrumGenerator is

-- stuff for spec gen interface, needs something actually doing withit

signal Data_Good : std_logic;
signal sink_eop : std_logic;
signal Buffered_Data : std_logic_vector (7 downto 0);



-- stuff added in when FFT added
signal DataValid : std_logic;

signal RealOutData : std_logic_vector (7 downto 0);
signal ImagOutData : std_logic_vector (7 downto 0);
signal RealExp : std_logic_vector (5 downto 0);

signal DataOutGood :  std_logic;

-- stuff added in when post processor added
signal ExpOut : std_logic_vector (5 downto 0);
signal DataGood : std_logic; -- LOOK OUT!!! there is also a Data_Good, must sort this out
signal SpecAmplitude_gash : std_logic_vector (8 downto 0);
signal SweepCount : std_logic_vector (7 downto 0);



component SliceBuffer is
port (
				Clock50 : in std_logic;  -- might get a faster ext clock later!
				nReset : in std_logic;	
				Clock4k : in std_logic;  --sample clock
				Clock1k : in std_logic;  --sample clock
				RawData : in std_logic_vector (7 downto 0); -- 
				DataGood : in std_logic; -- inform this entity that data is valid
				sink_eop : in std_logic; -- informing FFT is end of packet, and here to reset window index
			
				Buffered_Data : out std_logic_vector (7 downto 0)  -- the data after window applied
				
			);

end component;

component FFTController is

	port (
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		nReset : in std_logic;
		
		ECG_Data : in std_logic_vector(7 downto 0);
		ECG_Clock : in std_logic;
		DataValid : in std_logic;
		
		
		
		RealOutData : out std_logic_vector (7 downto 0);
		RealExp : out std_logic_vector (5 downto 0);
		
		ImagOutData : out std_logic_vector (7 downto 0);
		sink_eopx : out std_logic;
		
		DataOutGood : out std_logic

	);
end component;

component Samos is
port
(
      Clock50 : in std_logic;  -- might get a faster ext clock later!
		nReset : in std_logic;
		ECG_Clock : in std_logic;
	
		 DataGood : in std_logic;
		RealIn : in std_logic_vector (7 downto 0);
		ImagIn : in std_logic_vector (7 downto 0);
				
		ExpIn : in std_logic_vector (5 downto 0);
				
		ResultReady : out std_logic;
		ResultOut : out std_logic_vector (8 downto 0);
		ExpOut : out std_logic_vector (5 downto 0)
);
end component;


begin
DataValid<='1';
DataGood<='1';
SpecStrobe<=Clock4K;
--SpecSweep<='0';
--SpecAmplitude(7 downto 2) <= ExpOut; 
SpecAmplitude (7 downto 0) <= SpecAmplitude_gash(8 downto 1);
Debug1<=Buffered_Data(0);


SweepOut : process (SweepCount,nReset)
begin 
	if (nReset='0') then
		SpecSweep<='0';
	else
		if (SweepCount="11111111") then
			SpecSweep<='1';
		else 
			SpecSweep<='0';
		end if;
	end if;
end process;

SweepCounter: process (nReset,Clock4K,SweepCount)
begin
	if (nReset='0') then 
			SweepCount <=  "00000000";
	else if(rising_edge (Clock4K)) then
		SweepCount <= SweepCount + '1';
	end if;
	end if;
end process;

	
	
	
SliceBufferInst : SliceBuffer

port map

(
			Clock50 =>Clock50, 
			nReset =>nReset,
			Clock4k =>Clock4k ,
			Clock1k =>Clock1k ,
			RawData =>RawData,
			DataGood =>DataGood,
			sink_eop =>sink_eop,
			
			Buffered_Data =>Buffered_Data
				
			

);	

FFTControllerInst : FFTController
port map

(
		Clock50 => Clock50,
		nReset => nReset,
		
		ECG_Data =>  Buffered_Data,
		ECG_Clock =>  Clock4k,
		DataValid => DataValid,
		
		
		
		RealOutData => RealOutData,
		RealExp => RealExp,
		
		ImagOutData => ImagOutData,
		sink_eopx => sink_eop,
	
		DataOutGood => DataOutGood
);


SamosInst : Samos
port map
(
		Clock50 => Clock50 ,  
		nReset => nReset  ,
		ECG_Clock => Clock4k ,
		
		DataGood=> DataOutGood,
		RealIn => RealOutData ,
		ImagIn => ImagOutData ,
				
		ExpIn =>  RealExp,
				
	
		ResultOut  => SpecAmplitude_gash ,
		ExpOut => ExpOut 
);

end RTL;
