
LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 
entity SquareRoot is 

	port (
				Clock50 : in std_logic;  -- might get a faster ext clock later!
				nReset : in std_logic;	
				Square : in std_logic_vector (16 downto 0);  --sample clock	
				SquareStrobeIn : in std_logic; 
				
				Root	: out STD_LOGIC_VECTOR (8 DOWNTO 0); 
				RootReady : out std_logic
			);
		
end SquareRoot;


architecture RTL of SquareRoot is

component RootIteration is 
port (
				Clock50 : in std_logic;  -- might get a faster ext clock later!
				nReset : in std_logic;	
				StrobeIn : in std_logic;
				SquareIn : in std_logic_vector (16 downto 0);
				IterateRootIn : in std_logic_vector (8 downto 0);  
				
				SetThatBit : out std_logic; -- inform that that bit being tested is to 

				Finished : out std_logic; -- can terminate early
				StrobeOut : out std_logic -- inform is ready
			);
		
end component;


signal SquareReg : std_logic_vector (16 downto 0);

signal RootTempReg : std_logic_vector (8 downto 0);
signal RootReg : std_logic_vector (8 downto 0);

signal BitCount : integer range 0 to 8;


type StateType is (InReset,WaitData,LoadIteration,Iterate1,Iterate2,WaitResult,Process1,Process2,Process3,Process4,Done);
signal CurrentState : StateType;
signal NextState : StateType;

-- control signals to/from iterate block
signal IterateRoot : std_logic;
signal SetThatBit : std_logic;
signal Finished : std_logic;
signal IterationReady : std_logic;
signal RootDone : std_logic;
signal FlagTime : std_logic_vector( 4 downto 0);

begin 

-----------------------------------------------
-----------------------------------------------
	StateReset:process (nReset,Clock50)
	begin
		if (nReset ='0') then 
			CurrentState <= InReset;
	   elsif (rising_edge (Clock50)) then
			CurrentState <= NextState;
		end if;
	end process;
------------------------------------------------
------------------------------------------------
RootReady <= FlagTime(3);
	
--InReset,WaitData,LoadIteration,Iterate1,Iterate2,WaitResult,Process1,Process2,Done);	
process (CurrentState,SquareStrobeIn,IterationReady,BitCount)

begin

	case CurrentState is
	------------------------
		when InReset => 
----------------------
				NextState <= WaitData;
	 ------------------------
		when WaitData => 
				if (SquareStrobeIn = '1') then
					NextState <= LoadIteration;
				else 
					NextState <= WaitData;
				end if;
	 ------------------------
		when LoadIteration => 
				NextState <= Iterate1;
	 ------------------------
		when Iterate1 => 
				NextState <= Iterate2;
	 ------------------------
		when Iterate2 =>
			NextState <= WaitResult;	
    ------------------------
		when WaitResult => 
				if (IterationReady = '1') then
					NextState <= Process1;
				else 
					NextState <= WaitResult;
				end if;
 -----------------------
		when Process1 => 
			if ((BitCount = 0) or (Finished = '1')) then
				NextState <= Done;
			else 
				NextState <= Process2;
			end if;
		----------------------
	 ------------------------
		when Process2 => 
			
				NextState <= Process3;
			
		---------------------
		when Process3 =>
			NextState <= Process4;
		
		---------------------
		when Process4 =>
				NextState <= LoadIteration;
	  -------------------------
		when Done => 
			NextState <= WaitData;
		-------------------------
		-------------------------
			
		end case;
	
	end process;
-----------------------------------
-----------------------------------


process (CurrentState,Clock50)

begin

	if (rising_edge (Clock50)) then

		case CurrentState is
		------------------------
			when InReset => 
		----------------------
				--NextState <= WaitData;
	 ------------------------
		when WaitData => 
					RootDone <= '0';
					SquareReg<= Square;
					BitCount<= 8;
					RootReg <= "000000000";
					RootTempReg <= "100000000";
	 ------------------------
		when LoadIteration => 
				
				IterateRoot <= '1';
	 ------------------------
		when Iterate1 => 
				--NextState <= Iterate2;
	 ------------------------
		when Iterate2 =>
			IterateRoot <= '0';	
    ------------------------
		when WaitResult => 
 -----------------------
		when Process1 => 
			RootReg(BitCount)<= SetThatBit;
			
			
		----------------------
	   ----------------------
		when Process2 => 
			RootTempReg <= RootReg;
			if (BitCount > 0) then
					BitCount <= BitCount -1;
			end if;
			
		---------------------
	  -------------------------
	  when Process3 => 
			RootTempReg(BitCount) <= '1';
			
		---------------------
	  -------------------------
	  when Process4 => 
			
			
		---------------------
	  -------------------------
		when Done => 
			Root<= RootReg;
			RootDone<='1';
		-------------------------
		-------------------------
			
		end case;
	end if;
	
	
	end process;
-----------------------------------
-----------------------------------
AnswerReady : process (nReset,Clock50,SquareStrobeIn,RootDone)		
begin

		if ((nReset= '0') or (SquareStrobeIn= '1')) then
			FlagTime <= "00000";
		elsif  (RootDone = '1') then 
			FlagTime <= "00111";
		else 
			if (rising_edge (Clock50)) then
				if ((FlagTime < "10000") and (FlagTime > "00011")) then	
					FlagTime <= FlagTime + '1';
				end if;
			end if;
		end if;
		
end process;



RootIterationInst : RootIteration 
port map
(
	Clock50 => Clock50,
	nReset => nReset,	
	StrobeIn => IterateRoot,	
	
	SquareIn => SquareReg,	
	IterateRootIn => RootTempReg,	
				
	SetThatBit => SetThatBit,	
	Finished	=> Finished,											
	StrobeOut => IterationReady	
);





end RTL;