LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all; 

-- system emulator, 

entity SystemWrapper is

	port
	(
		Clock50	: in std_logic;
		Clock27 : in std_logic;
		nReset : in std_logic;
		Hold : in std_logic;  -- a button that freezes system
		--NullData : in std_logic;
		Debug0 : out std_logic; --JP1.1  PinA13
		Debug1 : out std_logic;	--JP1.2  PinB13
		Debug2 : out std_logic;	--JP1.3	PinA14
		Debug3 : out std_logic;	--JP1.4	PinB14
	--vga
			H_Sync  : out std_logic;
			V_Sync : out std_logic;
			RedOut : out std_logic_vector(3 downto 0);
			GreenOut : out std_logic_vector(3 downto 0);
			BlueOut : out std_logic_vector(3 downto 0)
	
	
	);
end SystemWrapper;

architecture RTL of SystemWrapper is


signal SpecClock : std_logic; -- in this model, the spectrum runs at 1/4 clock
	
signal Clock4K : std_logic;
signal Clock1K : std_logic;
signal ClockArb : std_logic;
signal RawData : std_logic_vector (7 downto 0);
signal ScopeHold : std_logic;


signal SpecAmplitude :  std_logic_vector (7 downto 0);
signal SpecStrobe :  std_logic;  -- strobe the value in. might not be totally necessary
signal SpecSweep :  std_logic;  -- probably only as an overide, should be automatic?
signal SpecReset :  std_logic; -- as opposed to global, might use for pause/restart
	
signal ScopeAmplitude :  std_logic;
signal ScopeStrobe :  std_logic;  -- strobe the value in. might not be totally necessary
signal ScopeSweep :  std_logic;  -- probably only as an overide, should be automatic?
signal ScopeReset :  std_logic; -- as opposed to global, might use for pause/restart




component SpectrumGenerator is
port
	(
		Clock50	: in std_logic;
		nReset : in std_logic;
		Clock4K : in std_logic;
		Clock1K : in std_logic;
		Hold : in std_logic;
		RawData : in std_logic_vector(7 downto 0);
		--NullData : in std_logic;
		-- currently just outputs stuff
		SpecAmplitude : out std_logic_vector (7 downto 0);
		SpecStrobe : out std_logic;  -- strobe the value in. might not be totally necessary
		SpecSweep : out std_logic;  -- probably only as an overide, should be automatic?
		SpecReset : out std_logic; -- as opposed to global, might use for pause/restart
		ScopeHold : out std_logic; -- so the spectrum can hold the scope back until 
										  -- some spectral data is on its way
	
		Debug1:out std_logic
	);
	
	
	
end component;

component ScopeSource is
port
	(
		Clock50	: in std_logic;
		nReset : in std_logic;
		Clock1K : in std_logic;
		Hold : in std_logic;
		ScopeHold : in std_logic;  -- I know!! a hold was already provisioned. Trust me!
		--NullData : in std_logic;
		-- currently just outputs stuff
		ScopeAmplitude : out std_logic;
		ScopeStrobe : out std_logic;  -- strobe the value in. might not be totally necessary
		ScopeSweep : out std_logic;  -- probably only as an overide, should be automatic?
		ScopeReset : out std_logic; -- as opposed to global, might use for pause/restart
		RawData : out std_logic_vector (7 downto 0)
	
	);
	
end component;




component ClockGen is
port
	(
		Clock50	: in std_logic;
		nReset : in std_logic;
	
		Clock4K : out std_logic;
		Clock1K : out std_logic;
		ClockArb : out std_logic
	);
	
end component;

component DisplaySystem is
port (
--global
			Clock50 : in std_logic;  -- might get a faster ext clock later!
			clock27 : in std_logic;
			nReset : in std_logic;
		--vga
			H_Sync  : out std_logic;
			V_Sync : out std_logic;
			RedOut : out std_logic_vector(3 downto 0);
			GreenOut : out std_logic_vector(3 downto 0);
			BlueOut : out std_logic_vector(3 downto 0);
		
		-- system interface
			--interface from rest of system  {TBA}
				-- system interface
			--interface from rest of system  {TBA}
			SpecAmplitude : in std_logic_vector (7 downto 0);
			SpecStrobe : in  std_logic;  -- strobe the value in. might not be totally necessary
			SpecSweep : in  std_logic;  -- probably only as an overide, should be automatic?
			SpecReset : in  std_logic; -- as opposed to global, might use for pause/restart
	
		--Scope
			ScopeAmplitude : in std_logic;
			ScopeStrobe : in std_logic;  -- strobe the value in. might not be totally necessary
			ScopeSweep : in std_logic;  -- probably only as an overide, should be automatic?
			ScopeReset : in std_logic -- as opposed to global, might use for pause/restart
		);
	end component;
	



begin

Debug0<=SpecStrobe;
--Debug1<=SpecSweep;
Debug2<=RawData(0);
Debug3<=SpecAmplitude(0);

SpecEm : SpectrumGenerator

port map

(

		Clock50	=> Clock50,-- in this model, the spectrum runs at 1/4 clock => Clock50,	
		nReset => nReset,
		Clock4K => Clock4K,
		Clock1K => Clock1K,
		Hold => Hold,
		RawData => RawData,
		--NullData => NullData,
		-- currently just outputs stuff
		SpecAmplitude => SpecAmplitude,
		SpecStrobe =>SpecStrobe ,
		SpecSweep =>SpecSweep ,
		SpecReset => SpecReset,
		ScopeHold => ScopeHold,
		
		Debug1=>Debug1

);

ScopeEm : ScopeSource

port map

(
		Clock50	=> Clock50,
		nReset => nReset,
		Clock1K => Clock1K,
		Hold => Hold,
		ScopeHold => ScopeHold,
		--NullData => NullData,
		ScopeAmplitude => ScopeAmplitude,
		ScopeStrobe => ScopeStrobe ,
		ScopeSweep => ScopeSweep ,
		ScopeReset => ScopeReset,
		RawData => RawData
		
);

ClockGenInst : ClockGen

port map

(
		Clock50,
		nReset,
	
		Clock4K,
		Clock1K,
		ClockArb
);


DisplySystemInst : DisplaySystem
port map

(
			Clock50, 
			clock27, 
			nReset, 
		--vga
			H_Sync, 
			V_Sync,
			RedOut,
			GreenOut,
			BlueOut,
		
		-- system interface
			--interface from rest of system  {TBA}
				-- system interface
			--interface from rest of system  {TBA}
			SpecAmplitude,
			SpecStrobe,  -- strobe the value in. might not be totally necessary
			SpecSweep, -- probably only as an overide, should be automatic?
			SpecReset, -- as opposed to global, might use for pause/restart
	
		--Scope
			ScopeAmplitude,
			ScopeStrobe, -- strobe the value in. might not be totally necessary
			ScopeSweep,  -- probably only as an overide, should be automatic?
			ScopeReset -- as opposed to global, might use for pause/restart
		);



end RTL;
