LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;

entity VgaCountersAndMux is 
	port 
	(
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		clock27 : in std_logic;
		nReset : in std_logic;
		
		SpecMux : in std_logic;
		ScopeMux : in std_logic;
		
		SpecRedOut : in std_logic_vector (3 downto 0);
		SpecGreenOut :in  std_logic_vector (3 downto 0);
		SpecBlueOut : in std_logic_vector (3 downto 0);
		
		ScopeRedOut : in std_logic_vector (3 downto 0);
		ScopeGreenOut :in  std_logic_vector (3 downto 0);
		ScopeBlueOut : in std_logic_vector (3 downto 0);
	
		H_Sync  : out std_logic;
		V_Sync : out std_logic;
		
		HorCount : out integer range 0 to 1023; 
      VertCount : out integer range 0 to 1023; 
		
		RedOut : out std_logic_vector(3 downto 0);
		GreenOut : out std_logic_vector(3 downto 0);
		BlueOut : out std_logic_vector(3 downto 0)
		
	);
	
	end VgaCountersAndMux;
	
architecture RTL of VgaCountersAndMux is
	

signal VGAClock : std_logic;
signal Hcount :  integer range 0 to 1023; 
signal Vcount :  integer range 0 to 1023; 

	
component VGAPLL IS
	PORT
	(
		inclk0 : IN STD_LOGIC;
		c0		: OUT STD_LOGIC 
	);
END component;	

begin


HorCount <= Hcount;
VertCount <= Vcount;
--horizontal count process
process (nReset,VGAClock)
	
begin
		if (nReset = '0') then
		Hcount <= 0;
		Vcount <= 0;
		
		elsif (rising_edge (VGAClock)) then
			Hcount <= Hcount+ 1;
			
			if Hcount = 799 then
				Vcount <= Vcount+1;
				Hcount <= 0;
			else
				if ((Vcount >= 525) and (Hcount >= 756)) then
					Vcount <= 0;
				end if;
			end if;	
		end if;
		
end process;
-- end H count process	

-- H synch process
process (nReset,Hcount)

begin

	if (nReset = '0') then
		H_Sync <= '1';
		
	else
	
		if ((Hcount >= 661) and (Hcount<= 756)) then
			H_Sync <= '0';
		else
			H_Sync <= '1';
		end if;
	
	end if;
	
end process;

--vertical count process



-- process for Vert synch
process (nReset,Vcount)

begin

	if (nReset = '0') then
		V_Sync <= '1';
	else
		if ((Vcount >= 491) and (Vcount <= 492)) then
			V_Sync <= '0';
		else
			V_Sync <= '1';
		end if;
	end if;
	
end process;

-- route the colours

process (nReset,SpecMux,ScopeMux)
begin
	if (nReset = '0') then
		RedOut <= "0000"; 
		GreenOut  <= "0000";
		BlueOut  <= "0000";
	
		elsif (SpecMux = '1' ) then
				RedOut <= SpecRedOut; 
				GreenOut  <= SpecGreenOut;
				BlueOut  <= SpecBlueOut;	
			elsif (ScopeMux = '1' ) then
				RedOut <= ScopeRedOut; 
				GreenOut  <= ScopeGreenOut;
				BlueOut  <= ScopeBlueOut;	
			else
				if (Hcount < 640 and Vcount < 480 ) then
				RedOut <= "0000"; 
				GreenOut  <= "1111";
				BlueOut  <= "0000";	
					else 
				RedOut <= "0000"; 
				GreenOut  <= "0000";
				BlueOut  <= "0000";
		
			end if;
			end if;
	
	
end process;


-----
-- 	
	
	
	
	----
-- vga pll instance
 TheVgaClock : VGAPLL

	port map 
	(

		inclk0 => clock27,
		c0 => VGAClock
	);
---------------------

end RTL;
