LIBRARY ieee ;
USE ieee.std_logic_1164.all ;
LIBRARY ieee ;
use ieee.std_logic_unsigned.all;

entity Vgablock is

	port (
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		clock27 : in std_logic;
		nReset : in std_logic;
	
		H_Sync  : out std_logic;
		V_Sync : out std_logic;
		RedOut : out std_logic_vector(3 downto 0);
		GreenOut : out std_logic_vector(3 downto 0);
		BlueOut : out std_logic_vector(3 downto 0);
		
		-- system interface
		SpecAmplitude : in std_logic_vector (7 downto 0);
			SpecStrobe : in  std_logic;  -- strobe the value in. might not be totally necessary
			SpecSweep : in  std_logic;  -- probably only as an overide, should be automatic?
			SpecReset : in  std_logic; -- as opposed to global, might use for pause/restart
	
	--Scope
			ScopeAmplitude : in std_logic;
			ScopeStrobe : in std_logic;  -- strobe the value in. might not be totally necessary
			ScopeSweep : in std_logic;  -- probably only as an overide, should be automatic?
			ScopeReset : in std_logic -- as opposed to global, might use for pause/restart
		
	);

end Vgablock;

architecture RTL of Vgablock is

signal HorCount : integer range 0 to 1023; 
signal VertCount : integer range 0 to 1023; 


--signal FreqSlot : std_logic_vector (4 downto 0);
--signal	Amplitude : std_logic_vector (7 downto 0);
--signal	Strobe :std_logic;  -- strobe the value in. might not be totally necessary
--signal	ColumnDone :  std_logic;  -- probably only as an overide, should be automatic?
--signal	ResetDisplay :  std_logic; -- as opposed to global, might use for pause/restart

-- interface to display 
signal FreqBin : std_logic_vector (4 downto 0);
--signal TimeSlot:std_logic_vector (7 downto 0);
signal Power : std_logic_vector (7 downto 0);
signal WritePower : std_logic;
signal DisplayReady : std_logic;

signal SpecRedOut : std_logic_vector (3 downto 0);
signal SpecGreenOut :  std_logic_vector (3 downto 0);
signal SpecBlueOut : std_logic_vector (3 downto 0);
signal SpecColourValid : std_logic;

signal ScopeRedOut : std_logic_vector (3 downto 0);
signal ScopeGreenOut : std_logic_vector (3 downto 0);
signal ScopeBlueOut: std_logic_vector (3 downto 0);
signal ScopeColourValid : std_logic;
--signal Scroll : std_logic;



component VgaCountersAndMux is 

	port 
	
	(
		Clock50 : in std_logic;  -- might get a faster ext clock later!
		clock27 : in std_logic;
		nReset : in std_logic;
		
		SpecMux : in std_logic;
		ScopeMux : in std_logic;
		
		SpecRedOut : in std_logic_vector (3 downto 0);
		SpecGreenOut :in  std_logic_vector (3 downto 0);
		SpecBlueOut : in std_logic_vector (3 downto 0);
		
		ScopeRedOut : in std_logic_vector (3 downto 0);
		ScopeGreenOut :in  std_logic_vector (3 downto 0);
		ScopeBlueOut : in std_logic_vector (3 downto 0);
	
		H_Sync  : out std_logic;
		V_Sync : out std_logic;
		
		HorCount : out integer range 0 to 1023; 
      VertCount : out integer range 0 to 1023; 
		
		RedOut : out std_logic_vector(3 downto 0);
		GreenOut : out std_logic_vector(3 downto 0);
		BlueOut : out std_logic_vector(3 downto 0)
		
	);
	end component;

	component SpectrumDisplayBox is 
    port (

	Clock50	: in std_logic;
	nReset : in std_logic;

	-- the input data is addressed like this,
	-- for easy 'face to math block
	-- concated to plain  address later
		SpecAmplitude : in std_logic_vector (7 downto 0);
		SpecStrobe : in  std_logic;  -- strobe the value in. might not be totally necessary
		SpecSweep : in  std_logic;  -- probably only as an overide, should be automatic?
		SpecReset : in  std_logic; -- as opposed to global, might use for pause/restart
	
	-----------------------------------------------
	-- interface to VGA - takes all the 
	-- Hcount and Vcount that is thrown 
	-- at it, but only returns data for relevant
	-- area so is quite custom
   Horizontal : in integer range 0 to 1023;  -- the same as the Hcount in block above
   Vertical : in integer range 0 to 1023;   -- the same as Vcount in block above
	-- dont forget, horizontal is actually counting the columns (timeslots)
	-- and vertical the rows 
	
	SpecRedOut : out std_logic_vector (3 downto 0);
	SpecGreenOut : out std_logic_vector (3 downto 0);
   SpecBlueOut : out std_logic_vector (3 downto 0);

	SpecColourValid : out std_logic
	);
	
	end component;
	
	
	component ScopeDisplayBox is 

port (

	   Clock50	: in std_logic;
		nReset : in std_logic;

	-- the input data is addressed like this,
	-- for easy 'face to math block
	-- concated to plain  address later
		ScopeAmplitude : in std_logic;
			ScopeStrobe : in std_logic;  -- strobe the value in. might not be totally necessary
			ScopeSweep : in std_logic;  -- probably only as an overide, should be automatic?
			ScopeReset : in std_logic; -- as opposed to global, might use for pause/restart
		
	-----------------------------------------------
	
	-- interface to VGA - takes all the 
	-- Hcount and Vcount that is thrown 
	-- at it, but only returns data for relevant
	-- area so is quite custom
		Horizontal : in integer range 0 to 1023;  -- the same as the Hcount in block above
		Vertical : in integer range 0 to 1023;   -- the same as Vcount in block above
	-- dont forget, horizontal is actually counting the columns (timeslots)
	-- and vertical the rows 
		ScopeRedOut : out std_logic_vector (3 downto 0);
		ScopeGreenOut : out std_logic_vector (3 downto 0);
		ScopeBlueOut : out std_logic_vector (3 downto 0);
		ScopeColourValid : out std_logic
	
	);
	
	end component;
	
	
begin

	

	--display memory container
	SpecBox : SpectrumDisplayBox
	
	port map
	(
	Clock50 => Clock50  ,
	nReset => nReset ,

	-- the input data is addressed like this,
	-- for easy 'face to math block
	-- concated to plain  address later
	SpecAmplitude => SpecAmplitude,
			SpecStrobe => SpecStrobe,
			SpecSweep => SpecSweep,
			SpecReset => SpecReset,
	-----------------------------------------------
	-- interface to VGA - takes all the 
	-- Hcount and Vcount that is thrown 
	-- at it, but only returns data for relevant
	-- area so is quite custom
	
	Horizontal => HorCount ,
	Vertical => VertCount ,
	
	
	SpecRedOut =>SpecRedOut  ,
	SpecGreenOut =>SpecGreenOut  ,
   SpecBlueOut => SpecBlueOut ,

	SpecColourValid => SpecColourValid 
	);

	
	ScopeBox : ScopeDisplayBox
	
	port map
	
	(
		Clock50 => Clock50,	
		nReset => nReset,

	
		ScopeAmplitude => ScopeAmplitude,
		ScopeStrobe => ScopeStrobe,  -- strobe the value in. might not be totally necessary
		ScopeSweep => ScopeSweep,  -- probably only as an overide, should be automatic?
		ScopeReset =>ScopeReset , -- as opposed to global, might use for pause/restart
	
		Horizontal => HorCount,  -- the same as the Hcount in block above
		Vertical => VertCount,   -- the same as Vcount in block above

		ScopeRedOut => ScopeRedOut,
		ScopeGreenOut => ScopeGreenOut,
		ScopeBlueOut => ScopeBlueOut,
		ScopeColourValid => ScopeColourValid 
	
	);

	-- VGA clocks and o/p mux
VgaClocks : VgaCountersAndMux 

port map 
(
		Clock50 => Clock50,
		clock27 => clock27 ,
		nReset => nReset,
		
		SpecMux => SpecColourValid ,
		ScopeMux => ScopeColourValid,
		
		SpecRedOut => SpecRedOut,
		SpecGreenOut => SpecGreenOut,
		SpecBlueOut =>SpecBlueOut ,
		
		ScopeRedOut => ScopeRedOut,
		ScopeGreenOut => ScopeGreenOut,
		ScopeBlueOut => ScopeBlueOut,
	
		H_Sync  => H_Sync ,
		V_Sync => V_Sync,
		
		HorCount => HorCount,
      VertCount => VertCount,
		
		RedOut => RedOut,
		GreenOut => GreenOut,
		BlueOut => BlueOut
);
		
end RTL;



